﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using Akqa.Method.CMS.DataAccess;
using Akqa.Method.CMS.BusinessEntities;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;

namespace CMS.Web.HttpHandlers
{
    public class GetN2FileHandler : HandlerBase 
    {

        private n2FileCacheUtility _fileCache = new n2FileCacheUtility(); 
        
        public override void ProcessRequest()
        {
            base.ProcessRequest();
        }

        protected override void OnProcessingRequest(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(AssetPath))
            {
                n2File file = _fileCache.GetFile(AssetPath);

                if (file == null)
                {
                    n2FileManager fileManager = new n2FileManager();
                    file = fileManager.GetFileByFullPath(AssetPath);
                    _fileCache.AddFile(AssetPath, file); 
                }
                
                if ((file != null) && (!file.IsDirectory))
                {
                    if (string.IsNullOrEmpty(ImageWidth) && string.IsNullOrEmpty(ImageHeight))
                        OutputFile(file);
                    else
                        OutputFile(ImageHeight, ImageWidth, file);
                }
            }
        }

        private void OutputFile(n2File file)
        {
            //axo change made to set client side caching ttl 
            Response.Cache.SetExpires(DateTime.Now.AddMinutes(15));
            Response.Cache.SetCacheability(HttpCacheability.Public);

            Response.ContentType = file.ContentType;
            Response.BinaryWrite(file.FileContent.ToArray());
        }

        public void OutputFile(string height,string width, n2File file)
        {
            Image thumbnail = null;
            int h = Convert.ToInt32(height);
            int w = Convert.ToInt32(width);

            //using(Stream s = new MemoryStream(file.FileContent.ToArray()))
            using (Image image = Image.FromStream(new MemoryStream(file.FileContent.ToArray())))
            {
                thumbnail = new System.Drawing.Bitmap(w, h);

                using (System.Drawing.Graphics g = System.Drawing.Graphics.FromImage(thumbnail))
                {
                    g.InterpolationMode = System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
                    g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
                    g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
                    g.CompositingQuality = System.Drawing.Drawing2D.CompositingQuality.HighQuality;

                    g.DrawImage(image, 0, 0, w, h);
                }
                Response.Cache.SetExpires(DateTime.Now.AddMinutes(15));
                Response.Cache.SetCacheability(HttpCacheability.Public);
                Response.ContentType = file.ContentType;
                thumbnail.Save(Response.OutputStream, ImageFormat.Jpeg);
            }
        }
        public string AssetPath
        {
            get
            {
                return HttpUtility.UrlDecode(GetString("AssetPath"));
            }
        }
        public string ImageHeight
        {
            get
            {
                return HttpUtility.UrlDecode(GetString("h"));
            }
        }
        public string ImageWidth
        {
            get
            {
                return HttpUtility.UrlDecode(GetString("w"));
            }
        }

    }
}
