﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;
using System.Collections.Specialized;
using System.IO;

namespace CMS.Web.N2File
{
    public class SqlSiteMapProvider : System.Web.SiteMapProvider 
    {
        private static readonly n2FileManager fileManager = new n2FileManager();

        private SqlSiteMapNode NewNode(n2File file)
        {
            return new SqlSiteMapNode(this, file);
        }

        public override SiteMapNode FindSiteMapNode(string fullPath)
        {
            SqlSiteMapNode smn = null;
            n2File file = fileManager.GetFileByFullPath(fullPath);
            if (file != null)
                smn = NewNode(file);
            return smn;
        }

        public override SiteMapNodeCollection GetChildNodes(SiteMapNode node)
        {
            n2File file = fileManager.GetFileByFullPath(node.Url);
            SiteMapNodeCollection nodes = new SiteMapNodeCollection();
            if ((file!=null) && (file.IsDirectory))
            {
                foreach (n2File d in fileManager.GetChildren(file, true))
                    nodes.Add(NewNode(d));
                foreach (n2File f in fileManager.GetChildren(file, false))
                    nodes.Add(NewNode(f));
            }
            return nodes;
        }

        public override SiteMapNode GetParentNode(SiteMapNode node)
        {
            string parentPath = GetParentPath(node.Url);
            n2File parent = fileManager.GetFileByFullPath(parentPath);
            return NewNode(parent);
        }


        protected override SiteMapNode GetRootNodeCore()
        {
            return NewNode(fileManager.GetRootFolder());
        }


        public static string GetParentPath(string path)
        {
            int i = path.LastIndexOf("/");
            if (i == -1)
                return path;
            else
            {
                return path.Substring(0, i);
            }
        }

        public static void AddDirectory(string path, string name)
        {
            fileManager.AddDirectory(path, name);
        }

        public static void Delete(string path)
        {
            fileManager.Delete(HttpUtility.UrlDecode(path));
        }

    }
}
